function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const chromeLauncher = require('chrome-launcher');
const CDP = require('chrome-remote-interface');
const EventEmitter = require('events');
const getPort = require('get-port');
const Semaphore = require('./semaphore');

const PAGE_SIZES = {
  'a0': { width: 33.1, height: 46.8 },
  'a1': { width: 23.4, height: 33.1 },
  'a2': { width: 16.5, height: 23.4 },
  'a3': { width: 11.7, height: 16.5 },
  'a4': { width: 8.3, height: 11.7 },
  'a5': { width: 5.8, height: 8.3 },
  'a6': { width: 4.1, height: 5.8 },
  'a7': { width: 2.9, height: 4.1 },
  'a8': { width: 2.0, height: 2.9 },
  'letter': { width: 8.5, height: 11 },
  'legal': { width: 8.5, height: 14 },
  'legder': { width: 17, height: 11 }
};
const CM_PER_INCH = 2.54;

/**
 * Options for constructing CrToPdf
 * @typedef {CtorOptions} Options
 * @property {string} [chromePath] - Path to installation of Chrome or Chromium
 */

/**
 * Options for generating a PDF.
 * @typedef {ConvertOptions} Options
 * @property {string} url - URL
 * @property {'portrait'|'landscape'} [orientaton] - Paper orientation
 * @property {boolean} [printBackground] - Print background graphics
 * @property {'a0'|'a1'|'a2'|'a3'|'a4'|'a5'|'a6'|'a7'|'a8'|'letter'|'legal'|'ledger'} [format] - Paper size
 * @property {number} [marginTop] - Top margin in cm
 * @property {number} [marginBottom] - Bottom margin in cm
 * @property {number} [marginLeft] - Left margin in cm
 * @property {number} [marginRight] - Right margin in cm
 * @property {string} [page] - Page ranges, e.g. "1-5, 8, 11-13"
 */

/**
 * Class for generating a PDF from a web page.
 * Requires Chrome to be installed on the machine.
 * @example
 * var fs = require('fs');
 * var CrToPdf = require('crtopdf');
 *
 * var pdf = new CrToPdf();
 *
 * pdf.init()
 * .then(function () {
 *   return pdf.convert({ url: 'https://en.wikipedia.org' });
 * })
 * .then(function (buf) {
 *   fs.writeFileSync('wikipedia.pdf', buf);
 * })
 * .then(function () {
 *   pdf.dispose();
 * });
 */
class CrToPdf extends EventEmitter {
  /**
   * Initializes CrToPdf
   * @param {CtorOptions} opts - Options
   */
  constructor(opts) {
    super();
    this.opts = opts || {};
    this.sem = new Semaphore(1);
  }

  /**
   * Initializes Chrome
   */
  init() {
    var _this = this;

    return _asyncToGenerator(function* () {
      _this.chrome = yield chromeLauncher.launch({
        port: yield getPort(),
        chromeFlags: ['--disable-gpu', '--headless'],
        chromePath: _this.opts.chromePath
      });
      _this.protocol = yield CDP({ port: _this.chrome.port });
      _this.connected = true;
      _this.protocol.on('disconnect', function () {
        _this.connected = false;
        _this.emit('disconnect');
      });
    })();
  }

  /**
   * Disposes Chrome
   */
  dispose() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      yield _this2.protocol.close();
      yield _this2.chrome.kill();
    })();
  }

  /**
   * Generates a PDF from a web page.
   * Do not call this method until the previous operation has finished.
   * @param {ConvertOptions} opts - Options
   * @return {Buffer} PDF data
   */
  convert(opts) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      if (!_this3.connected) {
        throw new Error('Not connected. Either init() wasn\'t called or Chrome crashed.');
      }

      yield _this3.sem.down();

      try {
        const { Page } = _this3.protocol;

        yield Page.enable();

        yield Page.navigate({ url: opts.url });

        yield new Promise(function (resolve, reject) {
          return _this3.protocol.once('Page.loadEventFired', resolve);
        });

        let pageSize = PAGE_SIZES[(opts.format || 'a4').toLowerCase()];
        let b64 = yield Page.printToPDF({
          landscape: !!opts.orientation && opts.orientation.toLowerCase() === 'landscape',
          printBackground: opts.printBackground,
          paperWidth: pageSize.width,
          paperHeight: pageSize.height,
          marginTop: opts.marginTop && opts.marginTop / CM_PER_INCH,
          marginBottom: opts.marginBottom && opts.marginBottom / CM_PER_INCH,
          marginLeft: opts.marginLeft && opts.marginLeft / CM_PER_INCH,
          marginRight: opts.marginRight && opts.marginRight / CM_PER_INCH,
          pageRanges: opts.pageRanges
        });

        return new Buffer(b64.data, 'base64');
      } finally {
        _this3.sem.up();
      }
    })();
  }
};

module.exports = CrToPdf;
//# sourceMappingURL=crtopdf.js.map